//
// zxdata.c
//
// Copyright (c) 2002 Michael Ringgaard. All rights reserved.
//
// ZignX Medical Audit Trail Logger
//


#include "zxlog.h"

struct data_device pc_port_device;
struct data_device instrument_port_device;
struct data_device keyboard_port_device;

void flush_data(struct data_device *dd)
{
  struct record *rec;

  if (dd->datalen == 0) return;

  rec = (struct record *) malloc(sizeof(struct record) + dd->datalen);
  rec->type = dd->rectype;
  rec->timestamp = dd->timestamp;
  rec->len = dd->datalen;
  memcpy(rec->data, dd->data, dd->datalen);

  dd->datalen = 0;
  dd->timestamp = 0;
  queue_log_record(rec, INFINITE);
}

void flush_all_data()
{
  // TODO: order in timestamp order
  flush_data(&pc_port_device);
  flush_data(&instrument_port_device);
  flush_data(&keyboard_port_device);
}

void __stdcall data_collector(void *arg)
{
  struct data_device *dd = (struct data_device *) arg;
  int rc;

  printf("Starting %s data collector\n", dd->name);
  while (1)
  {
    // Read data from port
    rc = read(dd->dev, dd->buffer, MAX_RECLEN - dd->datalen);
    //printf("%s: read %d\n", dd->name, rc);
    if (rc <= 0)
    {
      // Timeout, flush
      if (dd->datalen > 0) flush_all_data();
    }
    else
    {
      // Copy data
      if (dd->timestamp == 0) dd->timestamp = time();
      memcpy(dd->data + dd->datalen, dd->buffer, rc);
      dd->datalen += rc;

      // Flush if buffer is full
      if (dd->datalen == MAX_RECLEN) flush_all_data();
    }
  }
}

int init_data_logger()
{
  int f;
  int kbd_timeout;

  // Open PC data port
  printf("Opening PC data port\n");
  f = openport("pc-port");
  if (f < 0)
  {
    printf("Error %d opening PC data port\n", f);
    return f;
  }

  pc_port_device.name = "pc";
  pc_port_device.dev = f;
  pc_port_device.rectype = RECTYPE_PCDATA;

  // Open instrument data port
  printf("Opening instrument data port\n");
  f = openport("instrument-port");
  if (f < 0)
  {
    printf("Error %d opening instrument port\n", f);
    return f;
  }

  instrument_port_device.name = "instrument";
  instrument_port_device.dev = f;
  instrument_port_device.rectype = RECTYPE_INSTRDATA;

  // Open keyboard data port
  printf("Opening keyboard data port\n");
  f = open(get_property(zxlcfg, "zxlog", "keyboard", "/dev/console"), 0);
  if (f < 0)
  {
    printf("Error %d opening keyboard port\n", f);
    return f;
  }

  // Set keyboard timeout
  kbd_timeout = get_numeric_property(zxlcfg, "zxlog", "kbdtimeout", 4000);
  ioctl(f, IOCTL_SET_KEYTIMEOUT, &kbd_timeout, sizeof(kbd_timeout));

  keyboard_port_device.name = "keyboard";
  keyboard_port_device.dev = f;
  keyboard_port_device.rectype = RECTYPE_KEYBDATA;

  // Start data collectors
  pc_port_device.collector = beginthread(data_collector, 0, &pc_port_device, 0, NULL);
  instrument_port_device.collector = beginthread(data_collector, 0, &instrument_port_device, 0, NULL);
  keyboard_port_device.collector = beginthread(data_collector, 0, &keyboard_port_device, 0, NULL);

  return 0;
}
