//
// zxqueue.c
//
// Copyright (c) 2002 Michael Ringgaard. All rights reserved.
//
// FIFO queue
//

#include "zxlog.h"

struct queue *alloc_queue(int size)
{
  struct queue *q = (struct queue *) malloc(sizeof(struct queue));
  if (!q) return NULL;

  q->elems = (void **) malloc(size * sizeof(void *));
  if (!q->elems)
  {
    free(q);
    return NULL;
  }

  q->notfull = mksem(size);
  q->notempty = mksem(0);

  q->size = size;
  q->in = 0;
  q->out = 0;

  return q;
}

void free_queue(struct queue *q)
{
  close(q->notfull);
  close(q->notempty);
  free(q->elems);
  free(q);
}

int enqueue(struct queue *q, void *msg, unsigned int timeout)
{
  if (wait(q->notfull, timeout) < 0) return -ETIMEOUT;
  q->elems[q->in++] = msg;
  if (q->in == q->size) q->in = 0;
  semrel(q->notempty, 1);
  return 0;
}

void *dequeue(struct queue *q, unsigned int timeout)
{
  void *msg;

  if (wait(q->notempty, timeout) < 0) return NULL;
  msg = q->elems[q->out++];
  if (q->out == q->size) q->out = 0;
  semrel(q->notfull, 1);
  return msg;
}
